@@ -32,23 +32,25 @@ def argument_signature(callable_target):
         for name, param in dict(inspect.signature(callable_target).parameters).items()
     ]
 
-@lru_cache(None)
+cache_size = None
+
+@lru_cache(cache_size)
 def is_asyncgener(o):
     return inspect.isasyncgenfunction(o)
 
-@lru_cache(None)
+@lru_cache(cache_size)
 def isgeneratorfunction(o):
     return inspect.isgeneratorfunction(o)
 
-@lru_cache(None)
+@lru_cache(cache_size)
 def iscoroutinefunction(o):
     return inspect.iscoroutinefunction(o)
 
-@lru_cache(None)
+@lru_cache(cache_size)
 def isasyncgen(o):
     return inspect.isasyncgen(o)
 
-@lru_cache(None)
+@lru_cache(cache_size)
 def isgenerator(o):
     return inspect.isgenerator(o)
 
